@@ -3,37 +3,19 @@
 # GPIO Zero: a library for controlling the Raspberry Pi's GPIO pins
 #
 # Copyright (c) 2016-2021 Dave Jones <dave@waveform.org.uk>
+# Copyright (c) 2020 Fangchen Li <fangchen.li@outlook.com>
 # Copyright (c) 2016-2019 Ben Nuttall <ben@bennuttall.com>
 # Copyright (c) 2016 Andrew Scheller <github@loowis.durge.org>
 #
 # SPDX-License-Identifier: BSD-3-Clause
 
-from __future__ import (
-    unicode_literals,
-    print_function,
-    absolute_import,
-    division,
-)
-str = type('')
-
-
 from random import random
 from time import sleep
-from .mixins import ValuesMixin
-try:
-    from itertools import izip as zip
-except ImportError:
-    pass
 from itertools import cycle
-from math import sin, cos, pi
-try:
-    from statistics import mean
-except ImportError:
-    from .compat import mean
-try:
-    from math import isclose
-except ImportError:
-    from .compat import isclose
+from math import sin, cos, pi, isclose
+from statistics import mean
+
+from .mixins import ValuesMixin
 
 
 def _normalize(values):
